# frozen_string_literal: true

# Redmine - project management software
# Copyright (C) 2006-  Jean-Philippe Lang
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

module Redmine
  module Views
    class OtherFormatsBuilder
      def initialize(view)
        @view = view
      end

      def link_to(name, options={})
        url = {:format => name.to_s.downcase}.merge(options.delete(:url) || {}).except('page')
        caption = options.delete(:caption) || name
        html_options = {:class => name.to_s.downcase, :rel => 'nofollow'}.merge(options)
        @view.content_tag('span', @view.link_to(caption, url, html_options))
      end

      # Preserves query parameters
      def link_to_with_query_parameters(name, url={}, options={})
        params = @view.request.query_parameters.except(:page, :format).except(*url.keys)
        url = {:params => params, :page => nil, :format => name.to_s.downcase}.merge(url)

        caption = options.delete(:caption) || name
        html_options = {:class => name.to_s.downcase, :rel => 'nofollow'}.merge(options)
        @view.content_tag('span', @view.link_to(caption, url, html_options))
      end
    end
  end
end
